using System;

using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;

using Server;
using CommonTypes;

namespace ClientConsole
{
	/// <summary>
	/// Summary description for Client.
	/// </summary>
	class Client
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{

			TcpClientChannel channel = new TcpClientChannel();
			ChannelServices.RegisterChannel( channel );

			testSingleton();
			Console.WriteLine();
			testSingleCall();
			Console.WriteLine();
			testClientActivated();

			Console.Read();
		}

		static void testSingleton()
		{
			ServerMgr mgr = (ServerMgr)System.Activator.GetObject(
				typeof( ServerMgr ),
				"tcp://localhost:1234/ServerMgrSingleton" );
			testManager( mgr );

			Console.WriteLine();

			ServerMgr mgr2 = (ServerMgr)System.Activator.GetObject(
				typeof( ServerMgr ),
				"tcp://localhost:1234/ServerMgrSingleton" );
			testManager( mgr2 );
		}

		static void testSingleCall()
		{
			ServerMgr mgr = (ServerMgr)System.Activator.GetObject(
				typeof( ServerMgr ),
				"tcp://localhost:1234/ServerMgrSingleCall" );
			testManager( mgr );

			Console.WriteLine();

			ServerMgr mgr2 = (ServerMgr)System.Activator.GetObject(
				typeof( ServerMgr ),
				"tcp://localhost:1234/ServerMgrSingleCall" );
			testManager( mgr2 );
		}


		static void testClientActivated()
		{
			System.Runtime.Remoting.RemotingConfiguration.RegisterActivatedClientType(
				typeof( ServerMgr ),
				"tcp://localhost:1234" );

			ServerMgr mgr = new ServerMgr();
			testManager( mgr );

			Console.WriteLine();

			ServerMgr mgr2 = new ServerMgr();
			testManager( mgr2 );
		}
		
		static void testManager( ServerMgr mgr )
		{
			for ( int i = 0 ; i < 3 ; ++i )
			{
				try
				{
					Console.WriteLine( string.Format( "The Server Manager's count is {0}",
						mgr.GetCount() ) );
				}
				catch ( Exception ex )
				{
					Console.WriteLine( ex.Message );
				}
			}
			
			Counter counter = mgr.GetCounterObject();

			for ( int i = 0 ; i < 3 ; ++i )
			{
				try
				{
					Console.WriteLine( string.Format( "The Counter Object's count is {0}",
						counter.GetCount() ) );
				}
				catch ( Exception ex )
				{
					Console.WriteLine( ex.Message );
				}
			}
		}
	}
}
